#ifndef __TEXTAREA_H__
#define __TEXTAREA_H__

#include "Grid.H"
#include "Text.H"
#include "Widget.H"
#include <deque>
#include <pthread.h>

class TextArea:public Widget {
	public:
		TextArea(unsigned int wdth, unsigned int hght,
			unsigned int buf=0, bool bow=true, bool stripnl=false,
			bool readonly=true, int indent=0, char indent_char=0x20,
			bool pad=true);
		TextArea(const TextArea &tg);
		virtual ~TextArea();
		virtual widstat input(Char chr);
		virtual const Grid &draw();
		virtual bool update() { return (_changed); }
		virtual widstat resize(unsigned int wdth, unsigned int hght);
		virtual unsigned int height() const { return _height; }
		virtual unsigned int width() const { return _width; }
		virtual unsigned int lines() const { return _area.size(); }
		virtual unsigned int linepos() const { return _bufpos; }
		virtual widstat style(Char chr);
		virtual int scrollUp();
		virtual int scrollDown();
		virtual int pageUp();
		virtual int pageDown();
		virtual int toTop();
		virtual int toBottom();
		virtual int setIndent(int indent) { _indent=indent; return 0; }
		virtual bool stripNewlines(bool stripnl){ return ((_stripnl=stripnl)); }
		virtual bool breakOnWord(bool bow) { return ((_bow=bow)); }
		virtual Text getLine(unsigned int i) const { return _area[i]; }
		virtual Text operator[](unsigned int i) const { return getLine(i); }
		virtual TextArea &operator=(const TextArea &tg);
		virtual TextArea &operator=(const Text &t);
		virtual TextArea &operator=(Char chr);
		virtual TextArea &operator+=(const TextArea &tg);
		virtual TextArea &operator+=(const Text &t);
		virtual TextArea &operator+=(Char chr);
		virtual void clear();
	protected:
		unsigned int _width;
		unsigned int _height;
		unsigned int _bufsize;
		int _bufpos;
		Grid *_grd;
		Char _stile;
		bool _changed;
		bool _scrollbar;
		bool _bow;
		bool _stripnl;
		bool _readonly;
		int _indent;
	 	char _indent_char;
		bool _pad;
		deque<Text> _orig;
		deque<Text> _area;
		virtual int addToOrig(const Text &t);
		virtual int addToArea(const Text &t);
		virtual void fitToArea();
};

#endif // __TEXTAREA_H__
